"""DEPRECATED"""

napp = {
    # Mappings are dictionaries that map variable names to
    # functions that rewrite the value for a variable (for e.g. unit conversion)
    'mappings': {
        # format AGE to 3 positions
        'AGE': lambda x: x.zfill(3)
        # CITYPOP should be *1000
    },
    # Nocode is a list of variables whose values should not be coded (i.e. they should become literals)
    'nocode': ['BPLPARSE', 'CFU', 'COUNTYUS',
               'ENUMDIST', 'FAMUNIT', 'HEADLOC', 'HHNBRNO',
               'LINENUM', 'MOMLOC', 'NAMEFRST', 'NAMELAST', 'NHGISJOIN',
               'OCCSTRNG', 'PAGENUM', 'PARISHGB', 'PARSE', 'PERNUM',
               'POPLOC', 'QOCCGB', 'RECTYPE', 'REEL', 'RESLSNO',
               'SDSTCA', 'SEAUS', 'SERVANTS', 'SPLOC'],
    'integer': ['AGE', 'CFUSIZE', 'CITYPOP', 'CNTYAREA', 'ELDCH', 'HHWT',
                'NCOUPLES', 'NFAMS', 'NMOTHERS', 'NUMPERHH', 'OCSCORUS',
                'PERWT','REALPROP', 'RELATS', 'SEIUS', 'SERIAL', 'YEAR',
                'YNGCH', 'YRSUSA1']

    # technically OCSCORUS is not int but pretending  for now
    # should YEAR be an integer, a literal gYear or URI? => integer for now
}
canfam = {
    'mappings' : {
        'relhead2' : lambda x: x[0:3]
        # keep first 4 digits of relhead2 to match IPUMS/NAPP
    },
    'nocode': ['occ', 'hhdid', 'indlnm', 'indfnm', 'dwellid', 'chknote',
               'indnote', 'location'],
    'integer':['ageyr', 'magemo', 'moschool', 'urbpop', 'earnings', 'earnper', 'exearn']
}
mosaic = {
    'nocode': ['fname', 'lname', 'occupat'],
    'integer': ['age', 'hhsize', 'year']
}



from converter.qberify import build_schema
import argparse

parser = argparse.ArgumentParser(description="Convert CSV files to JSON Metadata files (according to W3C CSV on the Web)")
parser.add_argument('source_file', metavar='source_file', type=str, help="The input CSV file (without the .csv extension)")
parser.add_argument('dataset_name', metavar='dataset_name', type=str, help="A short name (slug) for the name of the dataset (will use input file name if not specified)")


if __name__ == '__main__':
    args = parser.parse_args()

    source_file = "{}.csv".format(args.source_file)
    target_file = "{}.json".format(args.source_file)

    build_schema(source_file, target_file, dataset_name=args.dataset_name)



from rdflib import Dataset, Graph, Namespace, RDF, RDFS, OWL, XSD, Literal, URIRef
import os
import yaml
import datetime
import string
import logging
import iribaker
import urllib
import uuid

from hashlib import sha1


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.INFO)
logger.addHandler(ch)

"""
Initialize a set of default namespaces from a configuration file (namespaces.yaml)
"""
# global namespaces
namespaces = {}
YAML_NAMESPACE_FILE = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'namespaces.yaml')


def init():
    """
    Initialize the module and assign namespaces to globals
    """
    # Read the file into a dictionary
    with open(YAML_NAMESPACE_FILE, 'r') as nsfile:
        global namespaces
        namespaces = yaml.load(nsfile)

    # Replace each value with a Namespace object for that value
    for prefix, uri in namespaces.items():
        if isinstance(prefix, str) and isinstance(uri, str):
            namespaces[prefix] = Namespace(uri)

    # Add all namespace prefixes to the globals dictionary (for exporting)
    for prefix, namespace in namespaces.items():
        globals()[prefix.upper()] = namespace

# Make sure the namespaces are initialized when the module is imported
init()


def reindent(s, numSpaces):
    s = s.split('\n')
    s = [(numSpaces * ' ') + string.lstrip(line) for line in s]
    s = "\n".join(s)
    return s


def serializeTrig(rdf_dataset):
    turtles = []
    for c in rdf_dataset.contexts():
        if c.identifier != URIRef('urn:x-rdflib:default'):
            turtle = "<{id}> {{\n".format(id=c.identifier)
            turtle += reindent(c.serialize(format='turtle'), 4)
            turtle += "}\n\n"
        else:
            turtle = c.serialize(format='turtle')
            turtle += "\n\n"

        turtles.append(turtle)

    return "\n".join(turtles)


def git_hash(data):
    """
    Generates a Git-compatible hash for identifying (the current version of) the data
    """
    s = sha1()
    s.update("blob %u\0" % len(data))
    s.update(data)
    return s.hexdigest()


def apply_default_namespaces(graph):
    """
    Apply a set of default namespaces to the RDFLib graph
    provided as argument and returns the graph.
    """

    for prefix, namespace in namespaces.items():
        graph.bind(prefix, namespace)

    return graph


def get_namespaces(base=None):
    """Return the global namespaces"""
    if base:
        namespaces['sdr'] = Namespace(unicode(base + u'/'))
        namespaces['sdv'] = Namespace(unicode(base + u'/vocab/'))
        with open(YAML_NAMESPACE_FILE, 'w') as outfile:
            yaml.dump(namespaces, outfile, default_flow_style=True)
    return namespaces


def safe_url(NS, local):
    """Generates a URIRef from the namespace + local part that is safe for
    use in RDF graphs

    Arguments:
    NS      -- a @Namespace object
    local   -- the local name of the resource
    """
    return URIRef(iribaker.to_iri(NS[local]))


def get_base_uri(dataset):
    """Get a base uri for the ``dataset`` (name)"""
    return Namespace('{}{}/'.format(namespaces['sdr'], dataset))


def get_value_uri(dataset, variable, value):
    """Generates a variable value IRI for a given combination of dataset, variable and value"""
    BASE = get_base_uri(dataset)

    return iribaker.to_iri(BASE['code/' + variable + '/' + value])


def get_variable_uri(dataset, variable):
    """Generates a variable IRI for a given combination of dataset and variable"""
    BASE = get_base_uri(dataset)

    return iribaker.to_iri(BASE[variable])


class DatastructureDefinition(Graph):
    """
    An RDFLib Graph that contains a datastructure definition, as specified by a QBer JSON dataset structure
    """

    def __init__(self, dataset_uri, dataset_name, variables):
        super(DatastructureDefinition, self).__init__()

        # Use the dataset_uri as BASE namespace
        BASE = Namespace("{}/".format(dataset_uri))

        # The URI of the DatastructureDefinition
        structure_uri = BASE['structure']

        self.add((dataset_uri, RDF.type, QB['DataSet']))
        self.add((dataset_uri, RDFS.label, Literal(dataset_name)))
        self.add((structure_uri, RDF.type, QB['DataStructureDefinition']))

        self.add((dataset_uri, QB['structure'], structure_uri))

        for variable_id, variable in variables.items():
            variable_uri = URIRef(variable['original']['uri'])
            variable_label = Literal(variable['original']['label'])
            variable_type = URIRef(variable['type'])


            # The variable as component of the definition
            component_uri = safe_url(BASE, 'component/' + variable['original']['label'])

            # Add link between the definition and the component
            self.add((structure_uri, QB['component'], component_uri))

            # Add label to variable
            # TODO: We may need to do something with a changed label for the variable
            self.add((variable_uri, RDFS.label, variable_label))

            if 'description' in variable and variable['description'] != "":
                self.add((variable_uri, RDFS.comment, Literal(variable['description'])))

            # If the variable URI is not the same as the original,
            # it is a specialization of a prior variable property.
            if variable['uri'] != str(variable_uri):
                self.add((variable_uri,
                          RDFS['subPropertyOf'],
                          URIRef(variable['uri'])))

            if variable_type == QB['DimensionProperty']:
                self.add((variable_uri, RDF.type, variable_type))
                self.add((component_uri, QB['dimension'], variable_uri))

                # Coded variables are also of type coded property (a subproperty of dimension property)
                if variable['category'] == 'coded':
                    self.add((variable_uri, RDF.type, QB['CodedProperty']))

            elif variable_type == QB['MeasureProperty']:
                # The category 'other'
                self.add((variable_uri, RDF.type, variable_type))
                self.add((component_uri, QB['measure'], variable_uri))
            elif variable_type == QB['AttributeProperty']:
                # Actually never produced by QBer at this stage
                self.add((variable_uri, RDF.type, variable_type))
                self.add((component_uri, QB['attribute'], variable_uri))

            # If this variable is of category 'coded', we add codelist and URIs for
            # each variable (including mappings between value uris and etc....)
            if variable['category'] == 'coded':
                codelist_uri = URIRef(variable['codelist']['original']['uri'])
                codelist_label = Literal(variable['codelist']['original']['label'])

                self.add((codelist_uri, RDF.type, SKOS['Collection']))
                self.add((codelist_uri, RDFS.label, Literal(codelist_label)))

                # The variable should point to the codelist
                self.add((variable_uri, QB['codeList'], codelist_uri))

                # The variable is mapped onto an external code list.
                # If the codelist uri is not the same as the original one, we
                # have a derived codelist.
                if variable['codelist']['uri'] != str(codelist_uri):
                    self.add((codelist_uri,
                              PROV['wasDerivedFrom'],
                              URIRef(variable['codelist']['uri'])))

                # Generate a SKOS concept for each of the values and map it to the
                # assigned codelist
                # But only if the 'values' are specified for this variable.
                if 'values' not in variable:
                    continue

                for value in variable['values']:
                    value_uri = URIRef(value['original']['uri'])
                    value_label = Literal(value['original']['label'])

                    self.add((value_uri, RDF.type, SKOS['Concept']))
                    self.add((value_uri, SKOS['prefLabel'], Literal(value_label)))
                    self.add((codelist_uri, SKOS['member'], value_uri))

                    # The value has been changed, and therefore there is a mapping
                    if value['original']['uri'] != value['uri']:
                        self.add((value_uri, SKOS['exactMatch'], URIRef(value['uri'])))
                        self.add((value_uri, RDFS.label, Literal(value['label'])))

            elif variable['category'] == 'identifier':
                # Generate a SKOS concept for each of the values
                # But only if the variable has specified values
                if 'values' not in variable:
                    continue

                for value in variable['values']:
                    value_uri = URIRef(value['original']['uri'])
                    value_label = Literal(value['original']['label'])

                    self.add((value_uri, RDF.type, SKOS['Concept']))
                    self.add((value_uri, SKOS['prefLabel'], value_label))

                    # The value has been changed, and therefore there is a mapping
                    if value['original']['uri'] != value['uri']:
                        self.add((value_uri, SKOS['exactMatch'], URIRef(value['uri'])))
                        self.add((value_uri, RDFS.label, Literal(value['label'])))

            elif variable['category'] == 'other':
                # Generate a literal for each of the values when converting the dataset (but not here)
                pass


class Profile(Graph):
    """
    An RDFLib Graph that contains author information based on a Google Profile
    """

    def __init__(self, profile):
        # A URI that represents the author

        # Virtuoso does not accept the @
        self.author_uri = SDP[urllib.quote_plus(profile['email'])]

        super(Profile, self).__init__(identifier=self.author_uri)

        self.add((self.author_uri, RDF.type, FOAF['Person']))
        self.add((self.author_uri, FOAF['name'], Literal(profile['name'])))
        self.add((self.author_uri, FOAF['email'], Literal(profile['email'])))
        self.add((self.author_uri, SDV['googleId'], Literal(profile['id'])))
        try:
            self.add((self.author_uri, FOAF['depiction'], URIRef(profile['image'])))
        except KeyError:
            logger.warning('No author depiction provided in author profile')



class Nanopublication(Dataset):
    """
    A subclass of the rdflib Dataset class that comes pre-initialized with
    required Nanopublication graphs: np, pg, ag, pig, for nanopublication, provenance,
    assertion and publication info, respectively.

    NOTE: Will only work if the required namespaces are specified in namespaces.yaml and the init() function has been called
    """

    def __init__(self, file_name):
        """
        Initialize the graphs needed for the nanopublication
        """
        super(Dataset, self).__init__()

        # Virtuoso does not accept BNodes as graph names
        self.default_context = Graph(store=self.store, identifier=URIRef(uuid.uuid4().urn))


        # Assign default namespace prefixes
        for prefix, namespace in namespaces.items():
            self.bind(prefix, namespace)

        # Get the current date and time (UTC)
        timestamp = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M")

        # Obtain a hash of the source file used for the conversion.
        # TODO: Get this directly from GitLab
        source_hash = git_hash(file(file_name).read())

        # Shorten the source hash to 8 digits (similar to Github)
        short_hash = source_hash[:8]

        # Determine a 'hash_part' for all timestamped URIs generated through this procedure
        hash_part = short_hash + '/' + timestamp

        # A URI that represents the version of the file being converted
        self.dataset_version_uri = SDR[source_hash]
        self.add((self.dataset_version_uri, SDV['path'], Literal(file_name, datatype=XSD.string)))
        self.add((self.dataset_version_uri, SDV['sha1_hash'], Literal(source_hash, datatype=XSD.string)))

        # ----
        # The nanopublication graph
        # ----
        name = (os.path.basename(file_name)).split('.')[0]
        self.uri = SDR[name + '/nanopublication/' + hash_part]


        # The Nanopublication consists of three graphs
        assertion_graph_uri = SDR[name + '/assertion/' + hash_part]
        provenance_graph_uri = SDR[name + '/provenance/' + hash_part]
        pubinfo_graph_uri = SDR[name + '/pubinfo/' + hash_part]

        self.ag = self.graph(assertion_graph_uri)
        self.pg = self.graph(provenance_graph_uri)
        self.pig = self.graph(pubinfo_graph_uri)

        # The nanopublication
        self.add((self.uri , RDF.type, NP['Nanopublication']))
        # The link to the assertion
        self.add((self.uri , NP['hasAssertion'], assertion_graph_uri))
        self.add((assertion_graph_uri, RDF.type, NP['Assertion']))
        # The link to the provenance graph
        self.add((self.uri , NP['hasProvenance'], provenance_graph_uri))
        self.add((provenance_graph_uri, RDF.type, NP['Provenance']))
        # The link to the publication info graph
        self.add((self.uri , NP['hasPublicationInfo'], pubinfo_graph_uri))
        self.add((pubinfo_graph_uri, RDF.type, NP['PublicationInfo']))

        # ----
        # The provenance graph
        # ----

        # Provenance information for the assertion graph (the data structure definition itself)
        self.pg.add((assertion_graph_uri, PROV['wasDerivedFrom'], self.dataset_version_uri))
        # self.pg.add((dataset_uri, PROV['wasDerivedFrom'], self.dataset_version_uri))
        self.pg.add((assertion_graph_uri, PROV['generatedAtTime'],
                     Literal(timestamp, datatype=XSD.dateTime)))

        # ----
        # The publication info graph
        # ----

        # The URI of the latest version of this converter
        # TODO: should point to the actual latest commit of this converter.
        # TODO: consider linking to this as the plan of some activity, rather than an activity itself.
        clariah_uri = URIRef('https://github.com/CLARIAH/wp4-converters')

        self.pig.add((self.uri, PROV['wasGeneratedBy'], clariah_uri))
        self.pig.add((self.uri, PROV['generatedAtTime'],
                      Literal(timestamp, datatype=XSD.dateTime)))


    def ingest(self, graph, target_graph=None):
        """
        Adds all triples in the RDFLib ``graph`` to this :class:`Nanopublication` dataset.
        If ``target_graph`` is ``None``, then the triples are added to the default graph,
        otherwise they are added to the indicated graph
        """
        if target_graph is None:
            for s, p, o in graph:
                self.add((s, p, o))
        else:
            for s, p, o in graph:
                self.add((s, p, o, target_graph))



import csv
import os
import rdflib
from iribaker import to_iri

YAML_NAMESPACE_FILE = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'namespaces.yaml')

FOAF = rdflib.Namespace('http://xmlns.com/foaf/0.1/')
LODE = rdflib.Namespace('http://linkedevents.org/ontology/')
IDS = rdflib.Namespace('http://data.socialhistory.org/resource/ids/')
TIME = rdflib.Namespace('https://www.w3.org/2006/time#')
OWLTIME = rdflib.Namespace("http://www.w3.org/TR/owl-time#")

rdf_graph = rdflib.Graph()

rdf_graph.bind('foaf', FOAF)
rdf_graph.bind('ids', IDS)
rdf_graph.bind('lode', LODE)
rdf_graph.bind('owltime', OWLTIME)

with open("indiv_001smpl.csv") as infile:
    reader = csv.reader(infile)
    header = reader.next()
    for row in reader:
        personid = row[2]
        day = str(row[9]).zfill(2)
        mnth = str(row[10]).zfill(2)
        year = str(row[11]).zfill(2)
        date = "{}-{}-{}".format(year, mnth, day)
        predicate = row[4]
        value = row[5]
        rdf_graph.add((IDS['event/' + personid + "_" + date], rdflib.RDF.type, LODE['Event']))
        rdf_graph.add((IDS['person/' + personid], rdflib.RDF.type, FOAF["Person"]))
        rdf_graph.add((IDS['event/' + personid + "_" + date], LODE['involvedAgent'], IDS['person/' + personid]))
        if day != "00" and mnth != "00":
            rdf_graph.add((IDS['event/' + personid + "_" + date], LODE['atTime'], rdflib.Literal(date, datatype=rdflib.XSD.date)))
        elif year != "00":
            rdf_graph.add((IDS['event/' + personid + "_" + date], LODE['atTime'], rdflib.Literal(str(year), datatype=rdflib.XSD.gYear)))
        else:
            rdf_graph.add((IDS['event/' + personid + "_" + date], LODE['atTime'], rdflib.Literal("unknown")))
        if "name" in predicate:
            rdf_graph.add((IDS['event/' + personid + "_" + date], IDS[predicate], rdflib.Literal(value)))
        else:
            rdf_graph.add((IDS['event/' + personid + "_" + date], IDS[predicate], IDS[predicate +'/' + value]))

with open("indiv_indiv_001smpl.csv") as infile:
    reader = csv.reader(infile)
    header = reader.next()
    for row in reader:
        personid1 = row[2]
        personid2 = row[3]
        day1 = str(row[8]).zfill(2)
        mnth1 = str(row[9]).zfill(2)
        year1 = str(row[10]).zfill(2)
        date1 = "{}-{}-{}".format(year1, mnth1, day1)
        day2 = str(row[11]).zfill(2)
        mnth2 = str(row[12]).zfill(2)
        year2 = str(row[13]).zfill(2)
        date2 = "{}-{}-{}".format(year2, mnth2, day2)
        predicate = row[5]
        # rdf_graph.add((IDS['event/' + personid1 + "_" + date], rdflib.RDF.type, IDS['event/' + personid1))
        rdf_graph.add((IDS['event/' + personid1 + "_" + date1 + "_" + date2], IDS[predicate], IDS['event/' + personid2]))
        # rdf_graph.add((IDS['event/' + personid + "_" + date1 + "_" + date2], rdflib.RDF.type, LODE['Event']))
        rdf_graph.add((IDS['person/' + personid1], rdflib.RDF.type, FOAF["Person"]))
        rdf_graph.add((IDS['person/' + personid2], rdflib.RDF.type, FOAF["Person"]))
        rdf_graph.add((IDS['event/' + personid2 + "_" + date1 + "_" + date2], LODE['involvedAgent'], IDS['person/' + personid2]))
        if day1 != "00" and mnth1 != "00":
            rdf_graph.add((IDS['event/' + personid + "_" + date1 + "_" + date2], OWLTIME['hasBeginning'], rdflib.Literal(date, datatype=rdflib.XSD.date)))
        elif year1 != "00":
            rdf_graph.add((IDS['event/' + personid + "_" + date1 + "_" + date2], OWLTIME['hasBeginning'], rdflib.Literal(str(year), datatype=rdflib.XSD.gYear)))
        else:
            rdf_graph.add((IDS['event/' + personid + "_" + date1 + "_" + date2], OWLTIME['hasBeginning'], rdflib.Literal("unknown")))
        if day2 != "00" and mnth2 != "00":
            rdf_graph.add((IDS['event/' + personid + "_" + date1 + "_" + date2], OWLTIME['hasEnd'], rdflib.Literal(date, datatype=rdflib.XSD.date)))
        elif year2 != "00":
            rdf_graph.add((IDS['event/' + personid + "_" + date1 + "_" + date2], OWLTIME['hasEnd'], rdflib.Literal(str(year), datatype=rdflib.XSD.gYear)))
        else:
            rdf_graph.add((IDS['event/' + personid + "_" + date1 + "_" + date2], OWLTIME['hasEnd'], rdflib.Literal("unknown")))

# ditto for context and context_context and indiv_context

with open("henry_ids_test.ttl", "w") as outfile:
    rdf_graph.serialize(outfile, format="turtle")


from requests import get
from bs4 import BeautifulSoup
from rdflib import Namespace, Graph, Literal, XSD, RDF
import re, json
import argparse
import os


def RepresentsInt(s):
    if s != None:
        try:
            int(s)
            return True
        except ValueError:
            return False


def makesoup(url):
    html = get(url).text
    soup = BeautifulSoup(html, 'html5lib')
    return(soup)


def getlinks(soup, regex):
    fill = soup.find_all('a', {'href': re.compile(regex)})
    for i in range(len(fill)):
        fill[i] = str(fill[i].get('href'))
    return fill


def getcodes(soup):
    scripts = []
    for script in soup.find_all('script'):
        if re.search('json', str(script)):
            scripts.append(script)

    codes = []
    for script in scripts:
        for line in str(script).splitlines():
            if re.search('categories:', line):
                codes.append(line)


    return codes


def makegraph(codes, vrb_lnk, description):

    base = 'http://data.socialhistory.org/resource/napp/'
    vrb_name = re.sub('.*/', '', vrb_lnk)
    vrb_url_full = base + vrb_name + '/'
    NAPP = Namespace(vrb_url_full)
    SKOS = Namespace('http://www.w3.org/2004/02/skos/core#')

    g = Graph()

    g.bind('napp', NAPP)
    g.bind('skos', SKOS)
    g.add((NAPP[vrb_name], RDF.type, SKOS['Scheme']))
    g.add((NAPP[vrb_name], SKOS['definition'], Literal(description)))

    indentlist = []
    codelist = []

    for code in codes:
        cd = code['code']
        lbl = code['label']

        if cd is None:
            cd = re.sub('\s|:', '_', lbl)
            cd = re.sub('/', '_or_', cd)
            cd = re.sub('[(),.''""]', '', cd)

        codelist.append(cd)
        indentlist.append(code['indent'])

        g.add((NAPP[cd], RDF.type, SKOS['Concept']))
        g.add((NAPP[cd], SKOS['prefLabel'], Literal(lbl)))
        g.add((NAPP[cd], SKOS['inScheme'], NAPP[vrb_name]))

        if RepresentsInt(lbl):
            g.add((NAPP[cd], RDF.value, Literal(lbl, datatype=XSD.int)))

        elif RepresentsInt(cd):
            g.add((NAPP[cd], RDF.value, Literal(cd, datatype=XSD.int)))


        if ((len(set(indentlist)) > 1) and (indentlist[i] > 0)):
            parentloc = indentlist[::-1].index(indentlist[i] - 1)
            parent = codelist[::-1][parentloc]

            g.add((NAPP[cd], SKOS['broader'], NAPP[parent]))
            g.add((NAPP[parent], SKOS['narrower'], NAPP[cd]))

    return g

baseurl = 'https://www.nappdata.org/'
soup = makesoup('https://www.nappdata.org/napp-action/variables/group')
groups = getlinks(soup=soup, regex='group\?id')

codelist = []
vrblist = []
desclist = []


parser = argparse.ArgumentParser()
parser.add_argument('path', nargs='?', default="napp/", help='Path to store converted data')
args = parser.parse_args()


path = args.path
if not path.endswith('/'):
    path += '/'


if not os.path.exists(path):
        os.makedirs(path)


for group in groups:
    soup = makesoup(baseurl + group)
    vrbs = getlinks(soup, 'variables/[A-Z]')

    for vrb in vrbs:
        soup = makesoup(baseurl + vrb)
        print(baseurl + vrb)
        codes = getcodes(soup)

        if len(codes) > 0:
            ptrn = re.search('\[.*\]', codes[0])
            codes = json.loads(ptrn.group(0))

        codelist.append(codes)
        vrblist.append(vrb)
        description = soup.find(id='description_section').find('p').getText()
        desclist.append(description)

graphs = {}

for i, codes in enumerate(codelist):
    vrbname = re.sub('/.*/', '', vrblist[i])
    graphs[vrbname] = makegraph(codes, vrblist[i], desclist[i])


# save json as backup
with open('nappcodebook.json', 'w') as out:
    json.dump(codelist, out)

for vrb_name, graph in graphs.items():
    with open(path + vrb_name + '.ttl', 'w') as out:
        graph.serialize(out, format='turtle')



import os
import csv
import json
import logging
from util import SDR
from iribaker import to_iri

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


_RESOURCE_BASE = str(SDR)
_RESOURCE_URI_PATTERN = "{0}{{}}/{{}}".format(_RESOURCE_BASE)

def build_schema(infile, outfile, delimiter=',', quotechar='\"', dataset_name=None):
    """Builds a basic QBer-style schema (probably deprecated)"""
    
    if dataset_name is None:
        dataset_name = os.path.basename(infile)

    dataset_uri = to_iri(SDR[dataset_name])

    metadata = {
        "dataset": {
            "file": infile,
            "name": dataset_name,
            "uri": dataset_uri,
            "variables": {}
        }
    }

    with open(infile, 'r') as infile_file:
        r = csv.reader(infile_file, delimiter=delimiter, quotechar=quotechar)

        header = r.next()

        logger.debug(header)

        for variable in header:
            variable = variable.decode('utf-8')
            variable_iri = to_iri(_RESOURCE_URI_PATTERN.format('variable', variable))
            col = {
                "category": "identifier",
                "category_comment": "`category` can be one of identifier, coded or other",
                "description": "The variable '{}' as taken from the '{}' dataset.".format(variable, dataset_name),
                "label": variable,
                "uri": variable_iri,
                "original": {
                    "label": variable,
                    "uri": variable_iri
                },
                "type": "http://purl.org/linked-data/cube#DimensionProperty",
                "valueUrl": "{}/{{{}}}".format(variable_iri,variable),
                "datatype_REMOVEME": "Any XML Schema datatype, only applicable for variables of type `other`",
                "transform_REMOVEME": "Any body of a JavaScript function, that returns some value based on an input `v`, the actual value of a variable",
                "values_REMOVEME": [
                    {
                        "comment": "`values` is a list of variable values that has the form specified here",
                        "count": "The frequency of this value for this variable",
                        "label": "The value itself, used as Literal value or as label in case of `identifier` or `coded`",
                        "original": {
                            "label": "The original value, in case of a mapped/modified value",
                            "uri": "The original URI of the value (typically follows the `valueUrl` template)"
                        },
                        "uri": "The URI for the value, ignored in case of `other`"
                    }
                ]
            }

            metadata['dataset']['variables'][variable] = col

    with open(outfile, 'w') as outfile_file:
        outfile_file.write(json.dumps(metadata, indent=True))

    logger.info("Done")
    return



#!/usr/bin/env python
# -*- coding: iso-8859-15 -*-

from distutils.core import setup
import os
import sys

with open('requirements.txt') as f:
    required = f.read().splitlines()

cow_base = 'src/'
cow_data = [ root.replace(cow_base, '') + '/*' for root,dirs,files in os.walk(cow_base) ]
cow_version = '0.34'

setup(name = 'cow_csvw',
      version = cow_version,
      description = 'Batch converter of large CSVs into CSVW/RDF',
      author = 'Rinke Hoekstra, Kathrin Dentler, Auke Rijpma, Richard Zijdeman, Albert Meroño-Peñuela',
      author_email = 'albert.merono@vu.nl',
      url = 'https://github.com/CLARIAH/COW',
      download_url = 'https://github.com/CLARIAH/COW/archive/' + cow_version + '.tar.gz',
      packages = ['cow_csvw'],
      package_dir = {'cow_csvw': 'src'},
      package_data = {'cow_csvw': cow_data},
      entry_points={'console_scripts' : [ 'cow_tool = cow_csvw.csvw_tool:main' ]},
      keywords = ['csv', 'rdf', 'csvw'],
      install_requires=required
)



from converter import Converter
import argparse
import json
import os.path 
import config

parser = argparse.ArgumentParser(description="Convert CSV files to RDF Data Cube (in parallel)")
parser.add_argument('source_file', metavar='source_file', type=str, help="The QBer-style JSON file that gives the schema of the CSV")
parser.add_argument('target_file', metavar='target_file', type=str, default='output.nq', help="The filename to which the RDF will be written (NQuads-format)")
parser.add_argument('--stop', metavar='N', type=int, default=None, required=False, help="IGNORED Specifies at what iteration to stop (currently ignored)")
parser.add_argument('--processes', metavar='N', type=int, default=4, required=False, help="Specifies the number of processes to spawn (default = 4)")
parser.add_argument('--chunksize', metavar='N', type=int, default=1000, required=False, help="Specifies the number of lines from the CSV file to send to each process (lower numbers use less memory, but make things slower")
parser.add_argument('--numberedobservations', type=bool, default=True, required=False, help="IGNORED Use the sequence number of each line to generate the URI for the observation")


if __name__ == '__main__':
    args = parser.parse_args()
    dirname = os.path.dirname(os.path.realpath(args.source_file))
    
    with open(args.source_file) as dataset_file:
        dataset = json.load(dataset_file)

    author_profile = {
        'email': config.EMAIL,
        'name': config.NAME,
        'id': config.ID
    }
    
    c = Converter(dataset['dataset'], dirname, author_profile, target=args.target_file)
    c.setProcesses(args.processes)
    c.setChunksize(args.chunksize)

    c.convert()



import os
from os import listdir
from os.path import isfile, join
import csv
import logging
import multiprocessing as mp
import uuid
import mappings
import datetime
import json

import js2py

from iribaker import to_iri
from functools import partial
from itertools import izip_longest

from rdflib import Graph, Dataset, URIRef, Literal

from util import Nanopublication, Profile, DatastructureDefinition, apply_default_namespaces, QB, RDF, XSD, SDV, SDR, PROV

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.INFO)
logger.addHandler(ch)


def grouper(n, iterable, padvalue=None):
    "grouper(3, 'abcdefg', 'x') --> ('a','b','c'), ('d','e','f'), ('g','x','x')"
    return izip_longest(*[iter(iterable)] * n, fillvalue=padvalue)


class Converter(object):
    """
    Converter configuration object for **QBer**-style conversion. Is used to set parameters for a conversion,
    and to initiate an actual conversion process (implemented in :class:`BurstConverter`)

    Takes a dataset_description (in QBer format) and prepares:

    * A dictionary for the :class:`BurstConverter` (either in one go, or in parallel)
    * A nanopublication structure for publishing the converted data (using :class:`converter.util.Nanopublication`)
    * A datastructure definition inside the nanopublication (using :class:`converter.util.DatastructureDefinition`)
    """


    def __init__(self, dataset, dirname, author_profile, source=None, target='output.nq'):
        """
        Initialization
        """

        # Defaults
        self._processes = 4          # Use 4 separate processes by default for converting CSV files
        self._chunksize = 1000       # Feed each process with 1000 lines at a time
        self._delimiter = ','        # Comma is the default delimiter
        self._quotechar = '\"'       # The double parenthesis is the default quoting character

        if source is None:
            self._source = os.path.join(dirname,  dataset['file'])
        else:
            self._source = source

        self._target = target

        self._dataset = dataset

        self.dataset_name = dataset['name']
        self.dataset_uri = URIRef(dataset['uri'])

        # For efficiency, convert the QBer-style value lists into dictionaries
        # But only for variables that have values, of course (see e.g. the use of valueUrl).
        self._variables = {}
        for variable, variable_definition in dataset['variables'].items():
            self._variables[variable] = variable_definition
            if 'values' in self._variables[variable]:
                self._variables[variable]['values_dictionary'] = dict([(unicode(v.get('label','')), v) for v in variable_definition['values']])

        # Initialize the nanopublication structure
        self.publication = Nanopublication(self._source)

        # Build profile information from the author profile provided
        self.addProfile(author_profile)

        # Build a datastructure definition based on the dataset description
        self.addDatastructureDefinition()

    def setDelimiter(self, delimiter):
        """Sets the delimiter for the CSV reader to ``delimiter`` """
        self._delimiter = delimiter

    def setQuotechar(self, quotechar):
        """Sets the quote character for the CSV reader to ``quotechar``"""
        self._quotechar = quotechar

    def setProcesses(self, processes):
        """Sets the number of processes to use for (parallel) conversion of the data"""
        self._processes = processes

    def setChunksize(self, chunksize):
        """Sets the number of lines to pass to each process"""
        self._chunksize = chunksize

    def setTarget(self, target):
        """Sets the output file to write the resulting RDF to (should be an N-Quads file)"""
        self._target = target

    def addProfile(self, author_profile):
        """Adds an author profile to the nanopublication"""

        print "Adding profile"
        # We add all triples from a Profile graph to the default graph of the nanopublication
        profile_graph = Profile(author_profile)
        self.publication.ingest(profile_graph)
        # We add an attribution relation between the nanopub assertion and the Profile author
        self.publication.pg.add((self.publication.ag.identifier, PROV['wasAttributedTo'], profile_graph.author_uri))
        self.publication.pig.add((self.publication.uri, PROV['wasAttributedTo'], profile_graph.author_uri))

    def addDatastructureDefinition(self):
        """Adds a datastructure definition to the nanopublication based on what we know about the current dataset"""

        print "Adding datastructure definition"
        # We add all triples from a DatastructureDefinition graph to the assertion graph of the nanopublication
        self.publication.ingest(DatastructureDefinition(self.dataset_uri, self.dataset_name, self._variables), self.publication.ag.identifier)

        # We link the dataset URI in the Provenance graph to the version of the dataset that was used in the conversion.
        self.publication.pg.add((self.dataset_uri, PROV['wasDerivedFrom'], self.publication.dataset_version_uri))

    def convert(self):
        """Starts the conversion process based on the parameters passed to the :class:``Converter`` initalization."""

        logger.info("Using {} processes".format(self._processes))

        # Open the target file
        with open(self._target, 'w') as target_file:
            # Write the nanopublication structure to the target file
            target_file.write(self.publication.serialize(format='nquads'))

            # Open the source file
            with open(self._source, 'r') as source_file:
                # Open the source file for reading as CSV
                reader = csv.reader(source_file,
                                    delimiter=self._delimiter,
                                    quotechar=self._quotechar,
                                    strict=True)

                # The headers are the first line (should correspond to variables)
                headers = reader.next()

                # TODO: Add check that headers and variables actually match!

                if self._processes > 1:
                    self._parallel(reader, headers, target_file)
                else:
                    self._simple(reader, headers, target_file)

    def _simple(self, reader, headers, target_file):
        """Starts a converter in a single process"""
        # Initialize the BurstConverter with the dataset and headers
        c = BurstConverter(self.publication.ag.identifier, self._dataset, self._variables, headers)
        # Out will contain an N-Quads serialized representation of the converted CSV
        out = c.process(0, reader, 1)
        # We then write it to the file
        target_file.write(out)

    def _parallel(self, reader, headers, target_file):
        """Starts the converter using multiple processes"""

        # Initialize a pool of processes (default=4)
        pool = mp.Pool(processes=self._processes)

        # The _burstConvert function is partially instantiated, and will be successively called with
        # chunksize rows from the CSV file
        burstConvert_partial = partial(_burstConvert,
                                       graph_identifier=self.publication.ag.identifier,
                                       dataset=self._dataset,
                                       variables=self._variables,
                                       headers=headers,
                                       chunksize=self._chunksize)

        # The result of each chunksize run will be written to the target file
        for out in pool.imap(burstConvert_partial,
                             enumerate(grouper(self._chunksize, reader))):
            target_file.write(out)

        # Make sure to close and join the pool once finished.
        pool.close()
        pool.join()


# This has to be a global method for the parallelization to work.
def _burstConvert(enumerated_rows, graph_identifier, dataset, variables, headers, chunksize):
    """The method used as partial for the parallel processing initiated in :func:`_parallel`."""

    count, rows = enumerated_rows
    c = BurstConverter(graph_identifier, dataset, variables, headers)

    print mp.current_process().name, count, len(rows)

    result = c.process(count, rows, chunksize)

    print mp.current_process().name, 'done'

    return result



class BurstConverter(object):
    """The actual converter, that processes the chunk of lines from the CSV file, and uses the instructions from the ``variables`` array to produce RDF."""


    _VOCAB_BASE = str(SDV)
    _RESOURCE_BASE = str(SDR)

    def __init__(self, graph_identifier, dataset, variables, headers):
        self._headers = headers
        self._variables = variables

        # TODO: Family is now superseded by a full dataset description in the form of QBer

        # if 'family' in config:
        #     self._family = config['family']
        #     try:
        #         family_def = getattr(mappings, config['family'])
        #         self._nocode = family_def['nocode']
        #         self._integer = family_def['integer']
        #         self._mappings = family_def['mappings']
        #     except:
        #         logger.warning('No family definition found')
        #         self._nocode = []
        #         self._integer = []
        #         self._mappings = {}
        # else:
        #     self._family = None

        # TODO: number_observations is now superseded by a full dataset description in the form of QBer

        # if 'number_observations' in config:
        #     self._number_observations = config['number_observations']
        # else:
        #     self._number_observations = None

        # TODO: stop is now superseded by a full dataset description in the form of QBer
        # self._stop = config['stop']

        # TODO: Now setting these as simple defaults
        self._family = None
        self._number_observations = True
        self._stop = None

        # TODO: Think of what to do here...
        if self._family is None:
            self._VOCAB_URI_PATTERN = "{0}{{}}/{{}}".format(self._VOCAB_BASE)
            self._RESOURCE_URI_PATTERN = "{0}{{}}/{{}}".format(self._RESOURCE_BASE)
        else:
            self._VOCAB_URI_PATTERN = "{0}{1}/{{}}/{{}}".format(self._VOCAB_BASE, self._family)
            self._RESOURCE_URI_PATTERN = "{0}{1}/{{}}/{{}}".format(self._RESOURCE_BASE, self._family)

        self.ds = apply_default_namespaces(Dataset())
        self.g = self.ds.graph(URIRef(graph_identifier))

        self._dataset_name = dataset['name']
        self._dataset_uri = URIRef(dataset['uri'])

    def process(self, count, rows, chunksize):
        """Process the ``rows`` read from the CSV file, and use ``count * chunksize`` to determine the absolute row number of the first row in ``rows``."""
        obs_count = count * chunksize
        for row in rows:
            # rows may be filled with None values (because of the izip_longest function)
            if row is None:
                continue

            if self._number_observations:
                observation_uri = self.resource('observation/{}'.format(self._dataset_name), obs_count)
            else:
                observation_uri = self.resource('observation/{}'.format(self._dataset_name), uuid.uuid4())

            self.g.add((observation_uri, QB['dataSet'], self._dataset_uri))

            index = 0
            for col in row:

                variable = self._headers[index]
                col = col.decode('utf-8')

                if len(col) < 1:
                    index += 1
#                     logger.debug('Col length < 1 (no value)')
                    continue
                # TODO: This applies a lambda function in the old version... but this is not currently
                #       supported.
                # elif variable in self._mappings:
                #     value = self._mappings[variable](col)
                elif variable in self._variables:
                    # The header string is indeed a variable (As it should be)

                    # What type of variable are we dealing with?
                    category = self._variables[variable]['category']

                    # The variable_uri is the URI specified in the variable definition
                    variable_uri = URIRef(self._variables[variable]['uri'])

                    # The original_variable_uri is the URI specified in the variable definition
                    original_variable_uri = URIRef(self._variables[variable]['original']['uri'])

                    try:
                        if col == "NA" or col == "N/A":
                            # TODO: Contentious... should these be ignored?
                            # Not a value... perhaps map this onto a default 'NA' uri?

                            # We take the default NA uri
                            value = SDR['NA']
                            # Add it to the graph
                            self.g.add((observation_uri, variable_uri, value))

                            # TODO: Add a flag to choose to preserve the original value or not.
                            # We take the original value from the CSV itself (not from the variable definition, as it often does not exist)
                            original_value = col
                            # Add it to the graph
                            self.g.add((observation_uri, original_variable_uri, Literal(original_value)))

                        elif category == "other":
                            # The variable is a literal

                            # If the variable has a transformation function, we apply it to the original value in the CSV file
                            # Note that this overwrites the provided value (i.e. the function overrides new values provided by QBer)
                            if 'transform_compiled' in self._variables[variable]:
                                # If we already have an evaluated version of the transformation function, use it
                                f = self._variables[variable]['transform_compiled']
                                value = f(col)
                            elif 'transform' in self._variables[variable]:
                                # Otherwise, we evaluate the function definition
                                f = js2py.eval_js("function f(v) {{ {} }}".format(self._variables[variable]['transform']))
                                self._variables[variable]['transform_compiled'] = f
                                value = f(col)
                            else:
                                # We take the 'label' (i.e. the potentially mapped value) from the
                                # corresponding value of the variable
                                value = self._variables[variable]['values_dictionary'][col]['label']

                            if 'datatype' in self._variables[variable]:
                                datatype = self._variables[variable]['datatype']
                                # If a datatype is specified, we use it to create the literal value
                                self.g.add((observation_uri, variable_uri, Literal(value, datatype=URIRef(datatype))))
                            else:
                                # Simply add the value to the graph without datatype
                                self.g.add((observation_uri, variable_uri, Literal(value)))

                            # TODO: Add a flag to choose to preserve the original value or not.
                            # We take the original 'label' from the corresponding value of the variable
                            original_value = self._variables[variable]['values_dictionary'][col]['original']['label']
                            # Add it to the graph
                            self.g.add((observation_uri, original_variable_uri, Literal(original_value)))

                        elif category == "coded" or category == "identifier":
                            # The variable is a URI

                            if 'valueUrl' in self._variables[variable]:
                                # If a valueUrl (taken from CSVW) is specified, we generate a URI
                                # for the value by applying the specified template to the column value

                                # The format args are key/value couples of header name and value
                                format_args = dict(zip(self._headers, [c.decode('utf-8') for c in row]))
                                value = to_iri(self._variables[variable]['valueUrl'].format(**format_args))

                            else:
                                # We take the 'uri' (i.e. the potentially mapped value) from the
                                # corresponding value of the variable
                                value = to_iri(self._variables[variable]['values_dictionary'][col]['uri'])

                            self.g.add((observation_uri, variable_uri, URIRef(value)))

                            if 'values_dictionary' in self._variables[variable] and col in self._variables[variable]['values_dictionary']:
                                # TODO: Add a flag to choose to preserve the original value or not.
                                # We take the original 'uri' from the corresponding value of the variable
                                original_value = to_iri(self._variables[variable]['values_dictionary'][col]['original']['uri'])
                                # Add it to the graph
                                self.g.add((observation_uri, original_variable_uri, URIRef(original_value)))
                        else:
                            print "Category {} unknown".format(category)

                    except KeyError:
                        pass
#                         print "Value found for variable {} does not exist in dataset description".format(variable)
#                         print col
#                         print row
#                         print

                elif variable == '':
                    # print "Empty variable name"
                    pass
                else:
                    # print "Could not find '{}' in defined variables, ignoring".format(variable)
                    pass

                index += 1

            obs_count += 1
            # if stop is not None and obs_count == stop:
            #     logger.info("Stopping at {}".format(obs_count))
            #     break

        # TODO: This ran inline SPARQL update queries. Need to reintegrate this with the current process.
        #
        # files = [f for f in listdir("update-queries/") if os.path.isfile(os.path.join("update-queries/", f))]
        # for f in files:
        #     if self._family in f and f.endswith("rq"):
        #         query = file("update-queries/" + f).read()
        #         self.g.update(query)

        return self.ds.serialize(format='nquads')

    def resource(self, resource_type, resource_name):
        """Produce a resource-URI based on the ``_RESOURCE_URI_PATTERN`` constant"""
        raw_iri = self._RESOURCE_URI_PATTERN.format(resource_type, resource_name)
        iri = to_iri(raw_iri)

        return URIRef(iri)

    def vocab(self, concept_type, concept_name):
        """Produce a vocab-URI based on the ``_VOCAB_URI_PATTERN`` constant"""
        raw_iri = self._VOCAB_URI_PATTERN.format(concept_type, concept_name)
        iri = to_iri(raw_iri)

        return URIRef(iri)
